<?php
/**
 * This File contains both the SteamSendAPIConnection Class and the StreamSendAPIResponse Class.
 *
 * @package PHP-StreamSend-Connector
 */

/**
 * Constant used when creating a connection object to set the format of the response object's data.
 * This value will return the xml as a string.
 */
define('STREAMSEND_API_TYPE_STRING',1);

/**
 * Constant used when creating a connection object to set the format of the response object's data.
 * This value will return the xml as a Simple XML Object.
 */
define('STREAMSEND_API_TYPE_SIMPLXML',2);

/**
 * This class handles connections to the StreamSend API Server.
 *
 * Usage example:
 * <code>
 *  $streamsend = new StreamSendAPIConnection($api_login_id,$api_key);
 *  $response = $streamsend->request('/audiences');
 *
 *  if((!count($response->errors)) && $response->status == 200)){
 *    $xmlObj = $response->data;
 *  }else{
 *    // Any errors generated by the Curl Library,
 *    // StreamSendAPI Server, or the SimpleXML parser
 *    // will be stored in the $response->errors array.
 *  }
 * </code>
 *
 * @package PHP-StreamSend-Connector
 */
class StreamSendAPIConnection{
  
  /**
   * @var Integer Sets the default format of the data in the response object. The allowed values are
   * STREAMSEND_API_TYPE_STRING: The raw xml received from the StreamSend API Server as a string,
   * STREAMSEND_API_TYPE_SIMPLXML: A SimpleXML Object The returned xml as parsed by PHP's simplexml_load_string function
   */
  public $response_parse_type;
  
  /**
   * @var String StreamSend API Login ID
   */
  public $login_id;
  
  /**
   * @var String StreamSend API Key
   */
  public $key;
  
  /**
   * @var Mixed Curl Handle
   */
  public $curl;
  
  /**
   * @var Integer Default audience_id for connection
   */
  public $audience_id;
  
  /**
   * @var String Stores the last uri requested.
   */
  public $uri;
  
  /**
   * @param String login_id StreamSend API Login ID
   * @param String key StreamSend API Key
   * @param Integer response_parse_type constant that specifies the format the response object's data property.
   * @param Integer audience_id if this value is set requested urls can contain /audiences/{audience_id}/people/123 and the audience_id value will be automatically inserted. If it is not defined this object will have to get that value from the API just before the first request. (Storing it locally save a round trip to the API server)
   */
  public function __construct($login_id,$key,$response_parse_type=STREAMSEND_API_TYPE_SIMPLXML,$audience_id=null){
    $this->login_id = $login_id;
    $this->key = $key;
    $this->response_parse_type = $response_parse_type;
    $this->audience_id = $audience_id;
    $this->init_curl();
  }
  
  /**
   * @param String $uri URI to request from the StreamSend API Server, relative to the API root (ie /audiences/)
   * @param String $method HTTP method of the request
   * @param Mixed $data XML string, associative array, or SimpleXMLElement Object representing the xml to send to the StreamSend API Server
   * @param Integer $response_parse_type If set will override the current response_parse_type of this StreamSendConnection object just for this request.
   */
  public function request($uri,$method='GET',$data=null,$response_parse_type=null){
    if(stripos($uri,'{audience_id}') !== false){
      $uri = str_replace('{audience_id}',$this->audience_id(),$uri);
    }
    $this->uri = $uri;
    curl_setopt($this->curl, CURLOPT_CUSTOMREQUEST, $method);
    curl_setopt($this->curl, CURLOPT_URL, 'https://app.streamsend.com'.$uri);
    if($data){
      if(is_array($data)){
        $data = $this->array_to_xml($data);
      }elseif(is_a($data,'SimpleXMLElement')){
        $data = $data->asXML();
      }
      curl_setopt($this->curl, CURLOPT_POSTFIELDS, $data);
    }    
    return new StreamSendResponse(curl_exec($this->curl),$this,$response_parse_type);
  }
  
  /**
   * Sets up a curl object to communicate with the StreamSend API Server
   */
  protected function init_curl(){
    $this->curl = curl_init();
    curl_setopt($this->curl, CURLOPT_USERPWD, "{$this->login_id}:{$this->key}");
    curl_setopt($this->curl, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($this->curl, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($this->curl, CURLOPT_HTTPHEADER, array('Accept: application/xml','Content-Type: application/xml'));
    curl_setopt($this->curl, CURLOPT_HEADER,true);
  }
  
  /**
   * Converts an associative array to a simple xml structure
   */
  public function array_to_xml($a){
    $lines = array();
    foreach($a as $k => $v){
      $line = '<'.$k.'>';
      if(is_array($v)){
        $line.="\n".$this->array_to_xml($v)."\n";
      }else{
        $line.=$v;
      }
      $lines[] = $line.'</'.$k.'>';
    }
    return implode("\n",$lines);
  }
  
  /**
   * This function returns the audience id. If this StreamSendAPIConnection object does not already know
   * the audience id it will get it from the API and store it so it can be automatically inserted in
   * subsequent calls to the API. ( A url containing {audience_id} will be automatically substituted )
   */
  public function audience_id(){
    if($this->audience_id){return $this->audience_id;}
    $r = $this->request('/audiences','GET',null,STREAMSEND_API_TYPE_SIMPLXML);
    if($audience_id = intval((String) $r->data->audience[0]->id)){
      $this->audience_id = $audience_id;
    }
    return $this->audience_id;
  }
  
}

/**
 * This class defines an object that encapsulates the response from the StreamSend API Server.
 *
 * Usage example:
 * <code>
 *  $streamsend = new StreamSendAPIConnection($api_login_id,$api_key);
 *  $response = $streamsend->request('/audiences');
 *
 *  if((!count($response->errors)) && $response->status == 200)){
 *    $xmlObj = $response->data;
 *  }else{
 *    // Any errors generated by the Curl Library,
 *    // StreamSendAPI Server, or the SimpleXML parser
 *    // will be stored in the $response->errors array.
 *  }
 * </code>
 * 
 * @package PHP-StreamSend-Connector
 */
class StreamSendResponse {
  
  /**
   * @var StreamSendAPIConnection StreamSend API Connection object that created this response object
   */
  public $conn;
  
  /**
   * @var String Raw HTTP Headers of the response.
   */
  public $raw_headers;
  
  /**
   * @var Array An associative array of HTTP headers keys and their values. To get this values the method headers() should be called.
   * since this array is not populated unless needed.
   */
  protected $_parsed_headers=null;
  
  /**
   * @var Array Array of connection, parsing errors, or api errors encountered.
   */
  public $errors=array();
  
  /**
   * @var Integer HTTP Status Code of response from the server
   */
  public $status;
  
  /**
   * @var Mixed Results of the query in the format specified by the STREAMSEND_API_TYPE setting of the connection object.
   */
  public $data;
  
  /**
   * @param String Full response including HTTP headers.
   */
  public function __construct($full_response,$conn,$response_parse_type=null){
    if(is_null($response_parse_type)){$response_parse_type = $conn->response_parse_type;}
    $this->conn = $conn;
    $this->status = curl_getinfo($conn->curl, CURLINFO_HTTP_CODE);
    if($full_response == false){
      $this->errors[] = 'CURL Error # '.curl_errno($this->conn->curl).': '.curl_error($this->conn->curl);
    }
    $t = explode("\r\n\r\n",$full_response);
    $this->raw_headers = $t[0];
    if(isset($t[1])){
      switch($response_parse_type){
        case STREAMSEND_API_TYPE_SIMPLXML:
          if(strlen(trim($t[1]))){
            if(function_exists('libxml_use_internal_errors')){
              libxml_use_internal_errors(true);
            }
            if($xml = simplexml_load_string($t[1])){
              $this->data = $xml;
            }else{
              if(function_exists('libxml_get_errors')){
                foreach(libxml_get_errors() as $error){
                  $this->errors[] = 'SimpleXML Parsing Error: '.$error->message.'( line: '.$error->line.', column: '.$error->column.' )';
                }
              }else{
                $this->errors[] = 'SimpleXML Parsing Error';
              }
            }
          }
          break;
        case STREAMSEND_API_TYPE_STRING:
        default:
          $this->data = $t[1];
          break;
      }
      if($this->status >= 400){
        // checking for errors returned in body of response
        if($response_parse_type == STREAMSEND_API_TYPE_SIMPLXML){
          $xml = $this->data;
        }else{
          if(strlen(trim($t[1]))){
            if(function_exists('libxml_use_internal_errors')){
              libxml_use_internal_errors(true);
            }
            $xml = simplexml_load_string($t[1]);
            if(function_exists('libxml_get_errors')){
              foreach(libxml_get_errors() as $error){
                $this->errors[] = 'SimpleXML Parsing Error: '.$error->message.'( line: '.$error->line.', column: '.$error->column.' )';
              }
            }
            
          }else{
            $this->errors[] = array_shift(explode("\n",$this->raw_headers));
          }
        }
        if($xml){
          foreach($xml->error as $e){$this->errors[]=(String) $e;}
        }
      }
    }
  }
  
  /**
   * Returns an array of HTTP headers that were sent as part of this response.
   */
  public function headers(){
    if(!is_null($this->_parsed_headers)){return $this->_parsed_headers;}
    $this->_parsed_headers = array();
    foreach(explode("\n",$this->raw_headers) as $line){
      $t = explode(':',$line,2);
      $this->_parsed_headers[trim($t[0])] = (isset($t[1])) ? trim($t[1]) : '';
    }
    return $this->_parsed_headers;
  }
}
?>